@@ -1,6 +1,7 @@
 from . import grammar
 from .parser import Parser
-
+from .state import state
+from os.path import dirname, join
 
 class NetfileMixin(object):
 
@@ -10,6 +11,7 @@ class NetfileMixin(object):
         self.namespace = ''
         self.subnetlists = {}
         self._anon = {}
+        self.dirname = None
 
     def _make_anon(self, kind):
         """Make identifier for anonymous component"""
@@ -38,6 +40,10 @@ class NetfileMixin(object):
         self.namespace = namespace
         return ret
 
+    def _directive(self, string, namespace=''):
+        
+        return self.parser.cpts.Directive(self, string, namespace)
+        
     def _parse(self, string, namespace=''):
         """The general form is: 'Name Np Nm symbol'
         where Np is the positive node and Nm is the negative node.
@@ -46,26 +52,18 @@ class NetfileMixin(object):
         to the negative node.
         """
 
-        string = string.strip('... ')
+        if string.startswith('...'):
+            string = string[3:].strip()
         
         if string == '':
-            return None            
-
-        if string[0] == ';':
-            # Strings starting with ;; are schematic options.
-            if hasattr(self, 'opts'):
-                self.opts.add(string[1:])
-            return None
-
-        if string[0:9] == '.include ':
+            pass
+        elif string[0:9] == '.include ':
             self._include(string)
             return None
-
-        if string[0:4] == '.pdb':
+        elif string[0:4] == '.pdb':
             import pdb; pdb.set_trace()
-            return None
 
-        cpt = self.parser.parse(namespace + string, self)
+        cpt = self.parser.parse(string, namespace, self)
         return cpt
 
     def add(self, string):
@@ -77,9 +75,14 @@ class NetfileMixin(object):
         to the negative node.
         """
 
+        # Switch context to capture new symbol definitions
+        if self.context is not None:
+            state.switch_context(self.context)
         self._add(string)
         self._invalidate()
-
+        if self.context is not None:        
+            state.restore_context()
+        
     def _add(self, string, namespace=''):
         """The general form is: 'Name Np Nm symbol'
         where Np is the positive node and Nm is the negative node.
@@ -100,15 +103,43 @@ class NetfileMixin(object):
             self._cpt_add(cpt)
         return cpt
 
+    def netfile_add(self, filename):
+        """Add the nets from file with specified filename"""
+
+        self.dirname = dirname(filename)
+        self._netfile_add(filename)
+    
     def _netfile_add(self, filename, namespace=''):
         """Add the nets from file with specified filename"""
 
+        netfile = None
+
         try:
-            file = open(filename, 'r')
+            netfile = open(filename, 'r')
         except:
-            file = open(filename + '.sch', 'r')            
+            pass
 
-        lines = file.readlines()
+        if netfile is None:
+            try:
+                netfile = open(filename + '.sch', 'r')
+            except:
+                pass
 
+        if netfile is None:
+            try:
+                netfile = open(join(self.dirname, filename), 'r')
+            except:
+                pass            
+
+        if netfile is None:            
+            raise FileNotFoundError('Could not open ' + filename)
+
+        lines = netfile.readlines()
+        netfile.close()
+
+        if self.context is not None:        
+            state.switch_context(self.context)        
         for line in lines:
             self._add(line, namespace)
+        if self.context is not None:                    
+            state.restore_context()
