@@ -1,12 +1,13 @@
 """
-Copyright 2014--2019 Michael Hayes, UCECE
+Copyright 2014--2020 Michael Hayes, UCECE
 """
 
 from __future__ import division
-from .expr import Expr
+from .expr import expr
 from .printing import latex, pretty
 from .schematic import Schematic
 from .circuit import Circuit
+from .state import state
 
 class Network(object):
 
@@ -35,7 +36,7 @@ class Network(object):
 
         modargs = []
         for arg in self.args:
-            arg = Expr(arg)
+            arg = expr(arg)
             modargs.append(arg)
         return modargs
 
@@ -62,9 +63,9 @@ class Network(object):
         argsrepr = ', '.join([pretty(arg) for arg in self._tweak_args()])
         return '%s(%s)' % (self.__class__.__name__, argsrepr)
 
-    def latex(self):
+    def latex(self, **kwargs):
 
-        argsrepr = ', '.join([latex(arg) for arg in self._tweak_args()])
+        argsrepr = ', '.join([latex(arg, **kwargs) for arg in self._tweak_args()])
         return '\\mathrm{%s}(%s)' % (self.__class__.__name__, argsrepr)
 
     @property
@@ -170,11 +171,11 @@ class Network(object):
            scale: schematic scale factor, default 1.0
            node_spacing: spacing between component nodes, default 2.0
            cpt_size: size of a component, default 1.5
-           oversample: oversampling factor for png or pdf files
+           dpi: dots per inch for png files
            help_lines: distance between lines in grid, default 0.0 (disabled)
            debug: True to display debug information
         """
-        
+
         if 'label_ids' not in kwargs:
             kwargs['label_ids'] = False
         if 'label_values' not in kwargs:
@@ -183,12 +184,12 @@ class Network(object):
             kwargs['label_nodes'] = False
         if 'draw_nodes' not in kwargs:
             kwargs['draw_nodes'] = 'connections'
-
+        
         self.sch.draw(filename=filename, **kwargs)
         
     @property
     def cct(self):
-        """Convert a Network object into a Circuit object for debugging."""
+        """Convert a Network object into a Circuit object."""
 
         if hasattr(self, '_cct'):
             return self._cct
@@ -200,6 +201,11 @@ class Network(object):
         self._cct = cct
         return cct
 
+    def circuit(self):
+        """Convert a Network object into a Circuit object."""
+
+        return self.cct
+
     @property    
     def initial_value_problem(self):
         return self.cct.initial_value_problem
@@ -218,11 +224,36 @@ class Network(object):
 
     @property    
     def is_causal(self):
-        return self.cct.is_causal    
+        return self.cct.is_causal
+
+    @property
+    def has_dc(self):
+        return self.cct.has_dc
+
+    @property    
+    def has_ac(self):
+        return self.cct.has_ac
+
+    @property    
+    def has_transient(self):
+        return self.cct.has_transient
 
     @property    
     def kinds(self):
         """Return list of transform domain kinds."""        
         return self.cct.kinds
-    
 
+    @property
+    def symbols(self):
+        """Return dictionary of symbols defined in the network."""
+        
+        return self.cct.symbols
+
+    @property
+    def all_symbols(self):
+        """Return dictionary of symbols defined in the network and global
+        symbols."""
+
+        symbols = self.symbols
+        symbols.update(state.global_context.symbols)
+        return symbols
