@@ -12,7 +12,7 @@ This module performs schematic drawing using circuitikz from a netlist.
 >>> sch.add('W 0.2 0.2; right')
 >>> sch.draw()
 
-Copyright 2014--2019 Michael Hayes, UCECE
+Copyright 2014--2020 Michael Hayes, UCECE
 """
 
 # Components are positioned using two graphs; one graph for
@@ -34,9 +34,13 @@ Copyright 2014--2019 Michael Hayes, UCECE
 #      of the nodes are assigned based on the distance along the
 #      longest path.
 
+# Strings starting with ;; are schematic options.  They are parsed in
+# netfile.py and added to the opts attribute of the netlist.  They get
+# passed to the draw method of the Schematic class.
+
 
 from __future__ import print_function
-from .latex import format_label
+from .latex import latex_format_label, latex_format_node_label
 from .expr import Expr
 from . import schemcpts
 import sympy as sym
@@ -52,20 +56,37 @@ import math
 __all__ = ('Schematic', )
 
 
-def display_matplotlib(filename):
+def display_matplotlib(filename, dpi=150, scale=2):
         
     from matplotlib.pyplot import figure, figaspect
     from matplotlib.image import imread
     
     img = imread(filename)
-    w, h = figaspect(img)
-    fig = figure(figsize=(w, h))
+    h, w, d = img.shape
+    width = scale * w / dpi
+    height = scale * h / dpi
+    fig = figure(figsize=(width, height))
     ax = fig.add_subplot(111)
     ax.imshow(img)
     ax.axis('off')
     fig.subplots_adjust(left=0.05, right=0.95, top=0.95, bottom=0.05)
 
 
+def png_image_size(filename):
+
+    import struct
+    
+    with open(filename, 'rb') as f:
+        header = f.read(25)
+
+    if (header[:8] != b'\211PNG\r\n\032\n' and (header[12:16] != b'IHDR')):
+        raise Exception('%s not a png image' % filename)        
+    w, h = struct.unpack('>LL', header[16:24])
+    width = int(w)
+    height = int(h)
+    return width, height
+
+    
 class SchematicOpts(Opts):
 
     def __init__(self):
@@ -76,9 +97,9 @@ class SchematicOpts(Opts):
              'label_ids': True,
              'label_nodes': 'primary',
              'scale' : 1.0,
+             'dpi' : 150,             
              'cpt_size' : 1.5,
              'node_spacing' : 2.0,
-             'append' : '',
              'help_lines' : 0.0,
              'style' : 'american'})
 
@@ -104,7 +125,7 @@ class EngFormat(object):
 
         value = self.value
         if value == 0:
-            return '0' + self.unit
+            return '0' + '\,' + r'\mbox{' + self.unit + r'}'
 
         m = math.log10(abs(value))
 
@@ -149,13 +170,18 @@ class Node(object):
         self._port = False
         self._count = 0
         parts = name.split('_')
-        self.rootname = parts[0] if name[0] != '_' else name
-        self.primary = len(parts) == 1
+        # Primary 1, 2, a, a_3.  Not primary _1, _2, _a, _a_3, 0_3, a_b_c
+        self.primary = (name[0] != '_' and len(parts) <= 2) and not (name[0].isdigit() and len(parts) != 1)
+
+        # See MX1.sch for example.
+        if '._' in name:
+            self.primary = False
+        
         self.elt_list = []
         self.pos = 'unknown'
         # Sanitised name
         self.s = name.replace('.', '@')
-        self.label = name
+        self.label = latex_format_node_label(self.name)
         self.labelpos = None        
         self.pin = False
         self.pinlabel = ''
@@ -192,7 +218,7 @@ class Node(object):
             self._port = True
 
         self.elt_list.append(elt)
-        if elt.type not in ('O', ):
+        if elt.type not in ('A', 'O'):
             self._count += 1
 
     @property
@@ -236,7 +262,7 @@ class Node(object):
             return self.count > 2
 
         if draw_nodes == 'primary':        
-            return '_' not in self.name
+            return self.primary
         
         raise ValueError('Unknown argument %s for draw_nodes' % draw_nodes)
 
@@ -246,6 +272,33 @@ class Node(object):
 
         return self._port or self.count == 1
 
+    def show_label(self, label_nodes):
+        
+        if self.label == '':
+            return False
+        
+        name = self.basename
+
+        # pins is for backward compatibility
+        if label_nodes in ('none', 'pins', 'false', False):
+            return False
+        elif label_nodes in ('all', 'true', True):
+            return True
+        elif label_nodes == 'alpha':
+            return self.primary and name[0].isalpha()
+        elif label_nodes == 'primary':
+            return self.primary
+
+        # handle label_nodes = '{1, 2}' etc.
+        if label_nodes[0] == '{' and label_nodes[-1] == '}':
+            label_nodes = label_nodes[1:-1]
+        labels = [foo.strip() for foo in label_nodes.split(',')]
+        return self.label in labels
+
+    def debug(self):
+        print(' %s @ (%s), count=%d, pin=%s' % (self.name, self.pos,
+                                                self.count, self.pin))
+    
     
 class Schematic(NetfileMixin):
 
@@ -253,14 +306,14 @@ class Schematic(NetfileMixin):
 
         self.elements = OrderedDict()
         self.nodes = {}
-        # Shared nodes (with same voltage)
-        self.snodes = {}
         self.hints = False
         self._init_parser(schemcpts)
         self.cpt_size = 1.2
         self.node_spacing = 2.0
         self.scale = 1.0
         self.dummy_node = 0
+        self.context = None
+        self.debug = False
 
         if filename is not None:
             self.netfile_add(filename)
@@ -317,13 +370,6 @@ class Schematic(NetfileMixin):
             # An auxiliary node can be made a proper node.
             node.auxiliary = False
         
-        vnode = node.rootname
-
-        if vnode not in self.snodes:
-            self.snodes[vnode] = []
-
-        if nodename not in self.snodes[vnode]:
-            self.snodes[vnode].append(nodename)
         return node
 
     def _cpt_add(self, cpt):
@@ -354,7 +400,6 @@ class Schematic(NetfileMixin):
             o = 'O %s %s; rotate=%s, size=%s' % (n1, n2, cpt.angle, cpt.size)
             self.add(o)
             
-
             # Rename nodes
             parts = cpt.net.split(' ')
             parts[1] = on1
@@ -387,7 +432,7 @@ class Schematic(NetfileMixin):
         if cpt.type in ('O', 'P', 'W') or id_label.find('#') != -1:
             id_label = None
 
-        if cpt.type in ('S', 'U'):
+        if cpt.type in ('S', 'SW', 'U'):
             value_label = ''
             
         if cpt.args != ():
@@ -431,8 +476,8 @@ class Schematic(NetfileMixin):
         # expression, or symbol.  If this is not specified, it
         # defaults to the component identifier.  Note, some objects
         # we do not want to label, such as wires and ports.
-        cpt.id_label = '' if id_label is None else format_label(id_label)
-        cpt.value_label = cpt.id_label if value_label is None else format_label(value_label)
+        cpt.id_label = '' if id_label is None else latex_format_label(id_label)
+        cpt.value_label = cpt.id_label if value_label is None else latex_format_label(value_label)
 
         if cpt.opts_string != '':
             self.hints = True
@@ -467,8 +512,8 @@ class Schematic(NetfileMixin):
         # distance from the root of the graph.  To centre components,
         # a reverse graph is created and the distances are averaged.
 
-        self.xgraph = Graph('horizontal', self.nodes)
-        self.ygraph = Graph('vertical', self.nodes)
+        self.xgraph = Graph('horizontal', self.nodes, self.debug)
+        self.ygraph = Graph('vertical', self.nodes, self.debug)
 
         # Use components in orthogonal directions as constraints.  The
         # nodes of orthogonal components get combined into a
@@ -477,7 +522,7 @@ class Schematic(NetfileMixin):
 
             if elt.offset != 0:
                 raise ValueError('offset field should be removed')
-            if elt.ignore:
+            if elt.directive or elt.ignore:
                 continue
             
             elt.xlink(self.xgraph)
@@ -487,7 +532,7 @@ class Schematic(NetfileMixin):
         # in the desired directions.
         # Note, this must be done after the linking step.
         for m, elt in enumerate(self.elements.values()):
-            if elt.ignore:
+            if elt.directive or elt.ignore:
                 continue            
             elt.xplace(self.xgraph)
             elt.yplace(self.ygraph)
@@ -503,35 +548,6 @@ class Schematic(NetfileMixin):
         for n, node in self.nodes.items():
             node.pos = Pos(xpos[n] * scale, ypos[n] * scale)
 
-    def _make_wires1(self, snode_list):
-
-        num_wires = len(snode_list) - 1
-        if num_wires == 0:
-            return []
-
-        wires = []
-
-        # TODO: remove overdrawn wires...
-        for n in range(num_wires):
-            n1 = snode_list[n]
-            n2 = snode_list[n + 1]
-            
-            wires.append(self._parse('W_ %s %s' % (n1, n2)))
-
-        return wires
-
-    def _make_wires(self):
-        """Create implicit wires between common nodes."""
-
-        wires = []
-
-        snode_dir = self.snodes
-
-        for m, snode_list in enumerate(snode_dir.values()):
-            wires.extend(self._make_wires1(snode_list))
-
-        return wires
-
     def _setup(self):
         # This is called before node positions are assigned.
 
@@ -541,17 +557,13 @@ class Schematic(NetfileMixin):
         for nodename, node in self.nodes.items():
             if node.ref is not None:
                 continue
-            if node.cptname is not None:
+            if node.cptname is not None and not node.implicit:
                 raise ValueError('Unreferenced pin connection %s for %s' % (node.name, node.elt_list))
 
-    def _postamble_add(self, s):
-        if s == '':
-            return s
-
-        return ' ' + s.replace('.', '@')
-                
     def _tikz_draw(self, style_args='', **kwargs):
 
+        self.debug = kwargs.pop('debug', False)
+        
         self._setup()
         
         self._positions_calculate()
@@ -561,8 +573,8 @@ class Schematic(NetfileMixin):
                 'transform shape',
                 '/tikz/circuitikz/bipoles/length=%.2fcm' % self.cpt_size]
         opts.append(style_args)
-        if 'preamble' in kwargs:
-            opts.append(kwargs.pop('preamble'))
+        if 'options' in kwargs:
+            opts.append(kwargs.pop('options'))
 
         if 'font' in kwargs:
             font = kwargs.pop('font')            
@@ -570,36 +582,51 @@ class Schematic(NetfileMixin):
         
         s = r'\begin{tikzpicture}[%s]''\n' % ', '.join(opts)
 
+        # Add preamble        
+        if 'preamble' in kwargs:
+            s += '  ' + kwargs.pop('preamble') + '\n'
+        
         help = float(kwargs.pop('help_lines', 0))
         color = kwargs.pop('color', 'blue')
         if help != 0:
             start = Pos(-0.5, -0.5) * self.node_spacing
             stop = Pos(self.width + 0.5, self.height + 0.5) * self.node_spacing
 
-            s += r'\draw[help lines, %s] (%s) grid [xstep=%s, ystep=%s] (%s);''\n' % (
+            s += r'  \draw[help lines, %s] (%s) grid [xstep=%s, ystep=%s] (%s);''\n' % (
                 color, start, help, help, stop)
 
         # Write coordinates.  TODO, not all coordinates are needed
-        # so those can be weeded out.
+        # so those can be weeded out to simplify the generated file.
         for n in self.nodes.values():
             s += r'  \coordinate (%s) at (%s);''\n' % (n.s, n.pos)
 
+        # keyword args for second pass
+        kwargs2 = kwargs.copy()
+            
         # Draw components
-        for m, elt in enumerate(self.elements.values()):
+        for elt in self.elements.values():
             if elt.ignore:
-                continue            
+                continue
+            if elt.directive:            
+                for key, val in elt.opts.items():
+                    kwargs[key] = val 
+
             s += elt.draw(**kwargs)
             s += elt.draw_nodes(**kwargs)
             s += elt.draw_pins()
 
-        # Add the labels
-        for m, elt in enumerate(self.elements.values()):
+        # Add the node labels
+        for elt in self.elements.values():
             if elt.ignore:
-                continue            
-            s += elt.draw_labels(**kwargs)            
+                continue                        
+            if elt.directive:
+                for key, val in elt.opts.items():
+                    kwargs2[key] = val
+            s += elt.draw_node_labels(**kwargs2)            
 
         # Add postamble
-        s += self._postamble_add(kwargs.pop('append', ''))
+        if 'postamble' in kwargs:
+            s += '  ' + kwargs.pop('postamble') + '\n'        
 
         s += r'\end{tikzpicture}''\n'
 
@@ -609,9 +636,8 @@ class Schematic(NetfileMixin):
 
         root, ext = path.splitext(filename)
 
-        debug = kwargs.pop('debug', False)
-        oversample = float(kwargs.pop('oversample', 2))
         style = kwargs.pop('style', 'american')
+        self.dpi = float(kwargs.pop('dpi', 150))        
         self.cpt_size = float(kwargs.pop('cpt_size', 1.2))
         self.node_spacing = float(kwargs.pop('node_spacing', 2.0))
         self.scale = float(kwargs.pop('scale', 1.0))
@@ -629,8 +655,8 @@ class Schematic(NetfileMixin):
         # For debugging when do not want to write to file
         nosave = kwargs.pop('nosave', False)
 
-        self.circuitikz_version = circuitikz_version()
-        if self.circuitikz_version is None:
+        self.circuitikz_date, self.circuitikz_version = circuitikz_version()
+        if self.circuitikz_date is None:
             raise RuntimeError('circuitikz is not installed')
 
         content = self._tikz_draw(style_args=style_args, **kwargs)
@@ -638,15 +664,19 @@ class Schematic(NetfileMixin):
         if nosave:
             return
 
-        if debug:
-            print('width = %d, height = %d, oversample = %d, cpt_size = %.2f, node_spacing = %.2f, scale = %.2f'
-                  % (self.width, self.height, oversample, 
+        if self.debug:
+            print('circuitikz version %s (%s)' % (self.circuitikz_version,
+                                                  self.circuitikz_date))
+            print('width=%d, height=%d, dpi=%d, cpt_size=%.2f, node_spacing=%.2f, scale=%.2f'
+                  % (self.width, self.height, self.dpi, 
                      self.cpt_size, self.node_spacing, self.scale))
-            print(self.nodes)
+            print('Nodes:')
+            for node in self.nodes.values():
+                node.debug()
             # print(self.xgraph.cnodes)
             # print(self.ygraph.cnodes)
 
-        if ext in ('.pytex', '.schtex'):
+        if ext in ('.pytex', '.schtex', '.pgf'):
             open(filename, 'w').write(content)
             return
 
@@ -654,7 +684,7 @@ class Schematic(NetfileMixin):
         template = ('\\documentclass[a4paper]{standalone}\n'
                     '\\usepackage{amsmath}\n'
                     '\\usepackage{circuitikz}\n'
-                    '\\usetikzlibrary{fit, shapes}\n'
+                    '\\usetikzlibrary{fit, shapes, arrows}\n'
                     '\\begin{document}\n%s\\end{document}')
         content = template % content
 
@@ -666,7 +696,7 @@ class Schematic(NetfileMixin):
 
         pdf_filename = tex_filename.replace('.tex', '.pdf')
         run_latex(tex_filename)
-        if not debug:
+        if not self.debug:
             latex_cleanup(tex_filename, pdf_filename)
 
         if not path.exists(pdf_filename):
@@ -678,19 +708,19 @@ class Schematic(NetfileMixin):
 
         if ext == '.svg':
             convert_pdf_svg(pdf_filename, root + '.svg')
-            if not debug:
+            if not self.debug:
                 remove(pdf_filename)
             return
 
         if ext == '.png':
-            convert_pdf_png(pdf_filename, root + '.png', oversample)
-            if not debug:
+            convert_pdf_png(pdf_filename, root + '.png', self.dpi)
+            if not self.debug:
                 remove(pdf_filename)
             return
 
         raise RuntimeError('Cannot create file of type %s' % ext)
 
-    def draw(self, filename=None, opts={}, **kwargs):
+    def draw(self, filename=None, **kwargs):
         """
         filename specifies the name of the file to produce.  If None,
         the schematic is displayed on the screen.
@@ -700,11 +730,13 @@ class Schematic(NetfileMixin):
         kwargs include:
            label_ids: True to show component ids
            label_values: True to display component values
-           draw_nodes: True to show all nodes, False to show no nodes, 
+           draw_nodes: True to show all nodes,
+             False or 'none' to show no nodes, 
              'primary' to show primary nodes,
              'connections' to show nodes that connect more than two components,
              'all' to show all nodes
-           label_nodes: True to label all nodes, False to label no nodes, 
+           label_nodes: True to label all nodes,
+             False or 'none' to label no nodes, 
              'primary' to label primary nodes,
              'alpha' to label nodes starting with a letter,
              'pins' to label nodes that are pins on a chip,
@@ -714,13 +746,35 @@ class Schematic(NetfileMixin):
            scale: schematic scale factor, default 1.0
            node_spacing: spacing between component nodes, default 2.0
            cpt_size: size of a component, default 1.5
-           oversample: oversampling factor for png or pdf files
+           dpi: dots per inch for png files
            help_lines: distance between lines in grid, default 0.0 (disabled)
            debug: True to display debug information
         """
 
+        # None means don't care, so remove.
+        for key in list(kwargs.keys()):
+            if kwargs[key] is None:
+                kwargs.pop(key)
+
+        # Remove options that may be overridden
+        for elt in self.elements.values():
+            for key in list(elt.opts.keys()):
+                if key in kwargs:
+                    elt.opts.remove(key)
+
+        # Default options
+        opts = SchematicOpts()
         for key, val in opts.items():
-            if key not in kwargs or kwargs[key] is None:
+            if key not in kwargs:            
+                kwargs[key] = val            
+
+        # Global options (at end of list for historical reasons)
+        for eltname in reversed(self.elements):
+            elt = self.elements[eltname]
+            if not elt.directive:
+                break
+            for key, val in elt.opts.items():
+                # val is a str
                 kwargs[key] = val
 
         def in_ipynb():
@@ -730,7 +784,7 @@ class Schematic(NetfileMixin):
 
                 kernapp = cfg['IPKernelApp']
 
-                # Check if processing ipynb file.
+                # Check if processing ipynb file for Jupyter notebook.
                 if 'connection_file' in kernapp:
                     return True
                 elif kernapp and kernapp['parent_appname'] == 'ipython-notebook':
@@ -777,18 +831,20 @@ class Schematic(NetfileMixin):
             #    when the ipynb file is loaded.
             # 2. The image metadata (width, height) is not stored
             #    when the ipynb file is written non-interactively.
+            width, height = png_image_size(pngfilename)
+            # width, height specify the image dimension in pixels
             display_png(Image(filename=pngfilename,
-                              width=self.width * 100, 
-                              height=self.height * 100))
+                              width=width, height=height))
             return
 
         if filename is None:
             filename = tmpfilename('.png')
             # Thicken up lines to reduce aliasing causing them to
             # disappear, especially when using pdftoppm.
-            self.tikz_draw(filename=filename, preamble='bipoles/thickness=2',
+            self.tikz_draw(filename=filename, 
+                           options='bipoles/thickness=2',
                            **kwargs)
-            display_matplotlib(filename)
+            display_matplotlib(filename, self.dpi)
             return
 
         self.tikz_draw(filename=filename, **kwargs)
