@@ -24,22 +24,22 @@ __all__ = ('StateSpace', )
 # V1 1 0 {10 * u(t)}
 # V1 1 0 ac 10
 #
-# In each case, we use v1(t) as the independent source when determing
+# In each case, we use v1(t) as the independent source when determining
 # the A, B, C, D matrices.  We can then substitute the known value at
 # the end.
 
+
 def _hack_vars(exprs):
-    """Substitute iCanon1(t) with iC(t) etc. provided
-    there is no iCanon2(t)."""
+    """Substitute i_Canon1(t) with i_C(t) etc. provided
+    there is no i_Canon2(t)."""
 
     for m, expr in enumerate(exprs):
-        for c in ('iV', 'iC', 'iL', 'vC'):
+        for c in ('i_V', 'i_C', 'i_L', 'v_C'):
             sym1 = sympify(c + 'anon1(t)')
             sym2 = sympify(c + 'anon2(t)')            
             if expr.has(sym1) and not expr.has(sym2):
                 expr = expr.subs(sym1, sympify(c + '(t)'))
                 exprs[m] = expr
-                
 
 class StateSpace(object):
     """This converts a circuit to state-space representation."""
@@ -67,12 +67,12 @@ class StateSpace(object):
             
             if isinstance(elt, L):
                 if sselt.name in cct.elements:
-                    raise ValueError('Name conflict %s, either rename the component or iprove the code!' % sselt.name)
+                    raise ValueError('Name conflict %s, either rename the component or improve the code!' % sselt.name)
 
                 inductors.append(elt)
             elif isinstance(elt, C):
                 if sselt.name in cct.elements:
-                    raise ValueError('Name conflict %s, either rename the component or iprove the code!' % sselt.name)
+                    raise ValueError('Name conflict %s, either rename the component or improve the code!' % sselt.name)
                 
                 capacitors.append(elt)
             elif isinstance(elt, (I, V)):
@@ -80,8 +80,7 @@ class StateSpace(object):
                 
         self.cct = cct
         self.sscct = sscct
-
-        # sscct can be analysed in the time domain since it has not
+        # sscct can be analysed in the time domain since it has no
         # reactive components.
         
         dotx_exprs = []
@@ -94,11 +93,12 @@ class StateSpace(object):
             if isinstance(elt, L):
                 # Inductors  v = L di/dt  so need v across the L
                 expr = sscct[name].v / elt.cpt.L
-                var = sscct[name].isc
+                var = -sscct[name].isc
                 x0 = elt.cpt.i0
             else:
                 # Capacitors  i = C dv/dt  so need i through the C
-                expr = sscct[name].i / elt.cpt.C
+                # The current is negated since it is from a source V_Cx
+                expr = -sscct[name].i / elt.cpt.C
                 var = sscct[name].voc
                 x0 = elt.cpt.v0
 
@@ -151,16 +151,17 @@ class StateSpace(object):
             for node in cct.node_list:
                 if node == '0':
                     continue
-                yexprs.append(self.sscct[node].v.subs(subsdict).expand())
-                y.append(Vt('vn%s(t)' % node))
+                yexprs.append(self.sscct[node].v.subs(subsdict).expand().expr)
+                # Note, this can introduce a name conflict
+                y.append(Vt('v_%s(t)' % node))
 
         if branch_currents:
             for name in cct.branch_list:
                 # Perhaps ignore L since the current through it is a
                 # state variable?
                 name2 = cpt_map[name]                    
-                yexprs.append(self.sscct[name2].i.subs(subsdict).expand())
-                y.append(It('i%s(t)' % name))                    
+                yexprs.append(self.sscct[name2].i.subs(subsdict).expand().expr)
+                y.append(It('i_%s(t)' % name))                    
 
         Cmat, b = sym.linear_eq_to_matrix(yexprs, *statesyms)
         D, b = sym.linear_eq_to_matrix(yexprs, *sourcesyms)
