@@ -34,8 +34,10 @@ class LcapyTester(unittest.TestCase):
 
         """
         a = L(10, 5)
-        self.assertEqual2(a.Z, 10 * s, "Z incorrect.")
-        self.assertEqual2(a.Y, 1 / (10 * s), "Y incorrect.")        
+        self.assertEqual2(a.Zs, 10 * s, "Zs incorrect.")
+        self.assertEqual2(a.Ys, 1 / (10 * s), "Ys incorrect.")
+        self.assertEqual2(a.Zw, 10 * j * omega, "Z incorrect.")
+        self.assertEqual2(a.Yw, 1 / (10 * j * omega), "Y incorrect.")                
         self.assertEqual2(a.Voc.s, -10 * 5, "Voc incorrect.")
         self.assertEqual2(a.Isc.s, -5 / s, "Isc incorrect.")        
 
@@ -45,8 +47,10 @@ class LcapyTester(unittest.TestCase):
         """
         a = C(10, 5)
 
-        self.assertEqual2(a.Z, 1 / (10 * s), "Z incorrect.")
-        self.assertEqual2(a.Y, 10 * s, "Y incorrect.")        
+        self.assertEqual2(a.Zs, 1 / (10 * s), "Zs incorrect.")
+        self.assertEqual2(a.Ys, 10 * s, "Ys incorrect.")
+        self.assertEqual2(a.Zw, 1 / (10 * j * omega), "Z incorrect.")
+        self.assertEqual2(a.Yw, 10 * j * omega, "Y incorrect.")                
         self.assertEqual2(a.Voc.s, 5 / s, "Voc incorrect.")
         self.assertEqual(a.Isc.s, 50, "Isc incorrect.")        
 
@@ -69,7 +73,8 @@ class LcapyTester(unittest.TestCase):
         a = L(10) + L(5)
         b = a.simplify()
         self.assertEqual2(b.L, 15, "L incorrect.")
-        self.assertEqual2(b.Z, 15 * s, "Z incorrect.")
+        self.assertEqual2(b.Zs, 15 * s, "Zs incorrect.")
+        self.assertEqual2(b.Zw, 15 * j * omega, "Z incorrect.")        
         self.assertEqual2(type(b), L, "type incorrect.")
 
     def test_C_series_C(self):
@@ -79,7 +84,8 @@ class LcapyTester(unittest.TestCase):
         a = C(10) + C(15)
         b = a.simplify()
         self.assertEqual2(b.C, 6, "C incorrect.")
-        self.assertEqual2(b.Z, 1 / (6 * s), "Z incorrect.")
+        self.assertEqual2(b.Zs, 1 / (6 * s), "Zs incorrect.")
+        self.assertEqual2(b.Zw, 1 / (6 * j * omega), "Z incorrect.")        
         self.assertEqual2(type(b), C, "type incorrect.")
 
     def test_Vdc_series_Vdc(self):
@@ -119,16 +125,16 @@ class LcapyTester(unittest.TestCase):
         """
         a = Vac(10) + R(5)
         self.assertEqual2(a.voc, Vac(10).voc, "voc incorrect.")
-        self.assertEqual(a.Voc.ac[omega], Phasor(10), "Voc incorrect.")
-        self.assertEqual(a.V.ac[omega], Phasor(10), "Voc incorrect.")
+        self.assertEqual(a.Voc.ac[omegasym], Phasor(10), "Voc incorrect.")
+        self.assertEqual(a.V.ac[omegasym], Phasor(10), "Voc incorrect.")
         self.assertEqual(a.I, 0, "I incorrect.")
         self.assertEqual(a.v(0), 10, "Voc incorrect.")                
         self.assertEqual(a.i(0), 0, "I incorrect.")                
 
         b = a.norton()
         self.assertEqual2(b.voc, Vac(10).voc, "voc incorrect.")
-        self.assertEqual(b.Voc.ac[omega], Phasor(10), "Voc incorrect.")
-        self.assertEqual(b.V.ac[omega], Phasor(10), "Voc incorrect.")
+        self.assertEqual(b.Voc.ac[omegasym], Phasor(10), "Voc incorrect.")
+        self.assertEqual(b.V.ac[omegasym], Phasor(10), "Voc incorrect.")
         self.assertEqual(b.I, 0, "I incorrect.")
         self.assertEqual(b.v(0), 10, "Voc incorrect.")                
         self.assertEqual(b.i(0), 0, "I incorrect.")        
@@ -138,7 +144,7 @@ class LcapyTester(unittest.TestCase):
 
         """
         a = R(10) + L(5)
-        self.assertEqual2(a.Z, 5 * s + 10, "Z incorrect.")
+        self.assertEqual2(a.Zs, 5 * s + 10, "Zs incorrect.")
 
     def test_R_parallel_R(self):
         """Lcapy: check R | R
@@ -157,7 +163,7 @@ class LcapyTester(unittest.TestCase):
         a = L(10) | L(15)
         b = a.simplify()
         self.assertEqual2(b.L, 6, "L incorrect.")
-        self.assertEqual2(b.Z, 6 * s, "Z incorrect.")
+        self.assertEqual2(b.Zs, 6 * s, "Zs incorrect.")
         self.assertEqual2(type(b), L, "type incorrect.")
 
     def test_C_parallel_C(self):
@@ -167,7 +173,7 @@ class LcapyTester(unittest.TestCase):
         a = C(10) | C(15)
         b = a.simplify()
         self.assertEqual2(b.C, 25, "C incorrect.")
-        self.assertEqual2(b.Z, 1 / (25 * s), "Z incorrect.")
+        self.assertEqual2(b.Zs, 1 / (25 * s), "Zs incorrect.")
         self.assertEqual2(type(b), C, "type incorrect.")
 
     def test_I_parallel_I(self):
@@ -244,3 +250,32 @@ class LcapyTester(unittest.TestCase):
         self.assertEqual2(a.isc, 10 * cos(omega * t), "AC incorrect.")
 
 
+    def test_CPE(self):
+        """Lcapy: check CPE
+
+        """
+        a = CPE(10, 1)
+
+        self.assertEqual2(a.Zs, 1 / (10 * s), "Zs incorrect.")
+        self.assertEqual2(a.Ys, 10 * s, "Ys incorrect.")
+        self.assertEqual2(a.Zw, 1 / (10 * j * omega), "Zw incorrect.")
+        self.assertEqual2(a.Yw, 10 * j * omega, "Yw incorrect.")                
+
+        a = CPE(10, 0)
+
+        self.assertEqual2(a.Zs, 1.0 / 10, "Zs incorrect.")
+        self.assertEqual2(a.Ys, 10, "Ys incorrect.")
+        self.assertEqual2(a.Zw, 1.0 / 10, "Zw incorrect.")
+        self.assertEqual2(a.Yw, 10, "Yw incorrect.")
+
+        a = CPE(10, 0.5)
+
+        self.assertEqual2(a.Ys, 10 * sqrt(s), "Ys incorrect.")
+
+    def simplify1(self):
+
+        a = I(0) + Y(1)
+        b = a.simplify()
+
+        self.assertEqual2(b._Z, None, "Z incorrect.")
+        self.assertEqual2(b._Y, 1, "Y incorrect.")                
