@@ -1,11 +1,38 @@
+"""This file performs transformations between domains.
+
+Copyright 2018--2019 Michael Hayes, UCECE
+
+"""
+
 from .sym import sympify, pi
 from .symbols import f, s, t, omega, jomega
 
 def transform(expr, arg, **assumptions):
+    """If arg is f, s, t, omega, jomega perform domain substitution,
+    otherwise perform substitution.
+
+    Note (5 * s)(omega) will fail since 5 * s is assumed not to be
+    causal and so Fourier transform is unknown.  However, Zs(5 *
+    s)(omega) will work since Zs is assumed to be causal.
+
+    """
 
     if isinstance(expr, arg.__class__) and not isinstance(expr, Super):
         return expr.subs(arg)
 
+    # Handle expr(t), expr(s), expr(f)
+    if arg is t:
+        return expr.time(**assumptions)
+    elif arg is s:
+        return expr.laplace(**assumptions)
+    elif arg is f:
+        if isinstance(expr, omegaExpr):
+            return expr.subs(2 * pi * f)
+        else:
+            return expr.fourier(**assumptions)    
+
+    # Handle expr(texpr), expr(sexpr), expr(fexpr).  For example,
+    # expr(2 * f).
     result = None 
     if isinstance(arg, tExpr):
         result = expr.time(**assumptions)
@@ -18,7 +45,7 @@ def transform(expr, arg, **assumptions):
             result = expr.fourier(**assumptions)
 
     if result is not None:
-        return result.subs(arg)
+        return result.__class__(result.subs(arg))
 
     sarg = arg
     try:
@@ -35,6 +62,8 @@ def transform(expr, arg, **assumptions):
     elif isinstance(arg, omegaExpr):
         if isinstance(expr, fExpr):
             result = expr.subs(omega / (2 * pi))
+        elif isinstance(expr, cExpr):
+            result = expr
         else:
             result = expr.fourier(**assumptions).subs(omega / (2 * pi))
     elif sarg.is_constant():
@@ -59,6 +88,7 @@ def call(expr, arg, **assumptions):
     return expr.subs(arg)
 
 
+from .cexpr import cExpr    
 from .fexpr import fExpr    
 from .sexpr import sExpr
 from .texpr import tExpr
